@@ -29,6 +29,7 @@ pub struct HNClient {
     client: ureq::Agent,
 }
 
+/// A macro to log the runtime of an expression
 macro_rules! log {
     ($e:expr, $desc:expr) => {{
         let time = std::time::SystemTime::now();
@@ -78,32 +79,27 @@ impl HNClient {
         );
 
         let (sender, receiver) = crossbeam_channel::bounded(32);
-        let client = self.clone();
 
         // loads first 5 comments to ensure the corresponding `CommentView` has data to render
-        Self::load_comments(&client, &sender, &mut ids, 5)?;
-        // used to test loading bar
-        // std::thread::sleep(std::time::Duration::from_secs(1000));
-        std::thread::spawn(move || {
+        self.load_comments(&sender, &mut ids, 5)?;
+        std::thread::spawn({
+            let client = self.clone();
             let sleep_dur = std::time::Duration::from_millis(1000);
-            while !ids.is_empty() {
-                if let Err(err) = Self::load_comments(&client, &sender, &mut ids, 5) {
-                    warn!("encountered an error when loading comments: {}", err);
-                    break;
+            move || {
+                while !ids.is_empty() {
+                    if let Err(err) = client.load_comments(&sender, &mut ids, 5) {
+                        warn!("encountered an error when loading comments: {}", err);
+                        break;
+                    }
+                    std::thread::sleep(sleep_dur);
                 }
-                std::thread::sleep(sleep_dur);
             }
         });
         Ok(receiver)
     }
 
     /// Load the first `size` comments from a list of comment IDs.
-    fn load_comments(
-        client: &HNClient,
-        sender: &CommentSender,
-        ids: &mut Vec<u32>,
-        size: usize,
-    ) -> Result<()> {
+    fn load_comments(&self, sender: &CommentSender, ids: &mut Vec<u32>, size: usize) -> Result<()> {
         let size = std::cmp::min(ids.len(), size);
         if size == 0 {
             return Ok(());
@@ -113,7 +109,7 @@ impl HNClient {
             .drain(0..size)
             .collect::<Vec<_>>()
             .into_par_iter()
-            .map(|id| match client.get_item_from_id::<CommentResponse>(id) {
+            .map(|id| match self.get_item_from_id::<CommentResponse>(id) {
                 Ok(response) => Some(response),
                 Err(err) => {
                     warn!("failed to get comment (id={}): {}", id, err);
@@ -141,8 +137,10 @@ impl HNClient {
             format!("get story (id={}) using {}", id, request_url)
         );
 
-        let stories: Vec<Story> = response.into();
-        Ok(stories.first().unwrap().clone())
+        match <Vec<Story>>::from(response).pop() {
+            Some(story) => Ok(story),
+            None => Err(anyhow::anyhow!("failed to get story with id {}", id)),
+        }
     }
 
     /// Get a list of stories matching certain conditions
@@ -299,6 +297,10 @@ impl HNClient {
         if output.status.success() {
             match serde_json::from_slice::<Article>(&output.stdout) {
                 Ok(mut article) => {
+                    // Replace a tab character by 4 spaces as it's possible
+                    // that the terminal cannot render the tab character.
+                    article.content = article.content.replace('\t', "    ");
+
                     article.url = url.to_string();
                     Ok(article)
                 }
